/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.IQ.PubSub
 *
 *
 */
ML.load("utils/observable.js");
ML.load("services/x/data.js");

/**
 * IQDisco
 *   "JEP0060 - Publish-Subscribe" implementation
 *
 * Constructor:
 *   IQPubSub(jabberService)
 *
 * Methods:
 *   createNode(where, nodeName);
 *   createAnonymousNode(where);
 *   publishItem(where, nodeName, tag);
 *   removeItem(where, nodeName, itemsIds, ...);
 *   getPendingSubscriptionRequest(from);
 *   getAffilations(from);
 *   subscribeNode(where, jid, nodeName);
 *   unsubscribeNode(where, jid, nodeName);
 *   getSubscriptionForm(from, jid, nodeName);
 *   sendSubscriptionForm(to, jid, nodeName, xdata);
 *   getItems(from, nodeName, maxItems);
 *   getItemsById(from, nodeName, ids, ...);
 *   getConfigurationForNode(where, nodeName);
 *   sendConfigurationForNode(where, nodeName, xdata);
 *   deleteNode(where, nodeName);
 *   purgeNode(where, nodeName);
 *   getEntities(from, nodeName);
 *   setEntities(where, nodeName, (entity) | (jid, affilation, subscription), ...);
 *
 * Events:
 *   onIQPubSubError(errorCode, errorText, id, from, tag)
 *   onIQPubSubCreateNode(id, from, nodeName)
 *   onIQPubSubSubscribe(id, from, jid, nodeName, affilation, subscription)
 *   onIQPubSubResult(id, from)
 *   onIQPubSubEventSetItem(id, from, nodeName, items)
 *   onIQPubSubEventDeleteItems(id, from, nodeName, itemsIds)
 *   onIQPubSubEventDeleteNode(id, from, nodeName)
 *   onIQPubSubSubscriptionForm(id, from, jid, nodeName, xdata)
 *   onIQPubSubConfigureForNodeForm(id, from, nodeName, xdata)
 *   onIQPubSubAffilations(id, from, entities)
 *   onIQPubSubEntities(id, from, entities)
 *
 */

function IQPubSub(jabberService)
{
    this.base = Observable;
    this.base();

    this.jabberService = jabberService;
}

IQPubSub.prototype =
{
    PUBSUB_NS: "http://jabber.org/protocol/pubsub",
    PUBSUBEVENT_NS: "http://jabber.org/protocol/pubsub#event",
    PUBSUBOWNER_NS: "http://jabber.org/protocol/pubsub#owner",

    __proto__: Observable.prototype,
    namespaceURI: [IQPubSub.prototype.PUBSUB_NS,
		   IQPubSub.prototype.PUBSUBEVENT_NS,
		   IQPubSub.prototype.PUBSUBOWNER_NS],
    msgNamespaceURI: [IQPubSub.prototype.PUBSUBEVENT_NS],

    createNode: function(where, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "create",
							"node", nodeName),
					     this);
    },

    createAnonymousNode: function(where)
    {
	this.createNode(where);
    },

    publishItem: function(where, nodeName, tag)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "publish",
							"node", nodeName).
					     addDom(tag),
					     this);
    },

    removeItem: function(where, nodeName)
    {
	var tag = DB.createElement(this.PUBSUB_NS, "pubsub");
	var items = tag.addElement(this.PUBSUB_NS, "retract",
				   "node", nodeName);
	var i, j;

	for (i = 2; i < arguments.length; i++)
	    if (arguments[i] instanceof Array)
		for (j = 0; j < arguments[i].length; j++)
		    items.addElement(this.PUBSUB_NS, "item",
				     "id", arguments[i][j]);
	    else
		items.addElement(this.PUBSUB_NS, "item",
				 "id", arguments[i]);

        return this.jabberService.core.iqTag(null, where, null, "set",
					     tag, this);
    },

    getPendingSubscriptionRequest: function(from)
    {
        return this.jabberService.core.messageTag(null, from, null,
					     DB.createElement(this.PUBSUBOWNER_NS, "x",
							      "action", "get-pending"));
    },

    getAffilations: function(from)
    {
        return this.jabberService.core.iqTag(null, from, null, "get",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "affiliations"),
					     this);
    },

    subscribeNode: function(where, jid, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "subscribe",
							"jid", jid,
							"node", nodeName),
					     this);
    },

    unsubscribeNode: function(where, jid, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "unsubscribe",
							"jid", jid,
							"node", nodeName),
					     this);
    },

    getSubscriptionForm: function(from, jid, nodeName)
    {
        return this.jabberService.core.iqTag(null, from, null, "get",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "options",
							"jid", jid,
							"node", nodeName),
					     this);
    },

    sendSubscriptionForm: function(to, jid, nodeName, xdata)
    {
        return this.jabberService.core.iqTag(null, to, null, "set",
					     xdata.createDOM(DB.createElement(this.PUBSUB_NS, "pubsub").
							     addElement(this.PUBSUB_NS, "options",
									"jid", jid,
									"node", nodeName)),
							     this);
    },

    getItems: function(from, nodeName, maxItems)
    {
        return this.jabberService.core.iqTag(null, from, null, "get",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "items",
							"node", nodeName
							"max_items", maxItems),
					     this);
    },

    getItemsById: function(from, nodeName)
    {
	var tag = DB.createElement(this.PUBSUB_NS, "pubsub");
	var items = tag.addElement(this.PUBSUB_NS, "items",
				   "node", nodeName);
	var i, j;

	for (i = 2; i < arguments.length; i++)
	    if (arguments[i] instanceof Array)
		for (j = 0; j < arguments[i].length; j++)
		    items.addElement(this.PUBSUB_NS, "item",
				     "id", arguments[i][j]);
	    else
		items.addElement(this.PUBSUB_NS, "item",
				 "id", arguments[i]);

        return this.jabberService.core.iqTag(null, from, null, "get",
					     tag, this);
    },

    getConfigurationForNode: function(where, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "get",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "configure",
							"node", nodeName),
					     this);
    },

    sendConfigurationForNode: function(where, nodeName, xdata)
    {
        return this.jabberService.core.iqTag(null, to, null, "set",
					     xdata.createDOM(DB.createElement(this.PUBSUB_NS, "pubsub").
							     addElement(this.PUBSUB_NS, "configure",
									"node", nodeName)),
							     this);
    },

    deleteNode: function(where, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "delete",
							"node", nodeName),
					     this);
    },

    purgeNode: function(where, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "set",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "purge",
							"node", nodeName),
					     this);
    },

    getEntities: function(from, nodeName)
    {
        return this.jabberService.core.iqTag(null, where, null, "get",
					     DB.createElement(this.PUBSUB_NS, "pubsub").
					     addElement(this.PUBSUB_NS, "entities",
							"node", nodeName),
					     this);
    },

    setEntities: function(where, nodeName, jid, affilation, subscription)
    {
	var tag = DB.createElement(this.PUBSUB_NS, "pubsub");
	var items = tag.addElement(this.PUBSUB_NS, "entities",
				   "node", nodeName);
	var i, j;

	for (i = 2; i < arguments.length; i += 3)
	    if (arguments[i] instanceof Array)
		for (j = 0; j < arguments[i].length; j += 3)
		    items.addElement(this.PUBSUB_NS, "entity",
				     "jid", arguments[i][j],
				     "affilation", arguments[i][j+1],
				     "subscription", arguments[i][j+2]);
	    else
		items.addElement(this.PUBSUB_NS, "entity",
				 "jid", arguments[i],
				 "affilation", arguments[i+1],
				 "subscription", arguments[i+2]);

        return this.jabberService.core.iqTag(null, from, null, "set",
					     tag, this);
    },

    onIQResponse: function(tag)
    {
	var pubsubTag;

        if (tag.getAttribute("type") == "error") {
            var errTag = tag.getElementsByTagName("error").item(0);

            this.fireEvent("onIQPubSubError",
			   errTag.getAttribute("code"),
			   domToText(errTag),
			   tag.getAttribute("id"),
			   tag.getAttribute("from"),
			   tag);
        } else if (tag.getAttribute("type") == "result")
	    if (pubsubTag = tag.getElementsByTagName("pubsub").item(0)) {
		switch (pubsubTag.firstChildElement.localName)
	    }
	    else
		this.fireEvent("onIQPubSubResult",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"),
			       tag);
	else
	    this.jabberService.core.iqError(tag.getAttribute("to"),
					    tag.getAttribute("from"),
					    tag.getAttribute("id"),
					    501,
					    "Not Implemented");
    },

    onIQHandler: function(tag)
    {
	this.jabberService.core.iqError(tag.getAttribute("to"),
					tag.getAttribute("from"),
					tag.getAttribute("id"),
					501,
					"Not Implemented");
    },

    onMsgHandler: function(tag)
    {
	var xTag = tag.getElementsByTagNameNS(this.PUBSUBEVENT_NS, "x").item(0);
	var ctag;

	if (xTag) {
	    ctag = xTag.firstChildElement;

	    switch (ctag.localName) {
	    case "items":
		switch (ctag.firstChildElement.localName) {
		case "item":
		    this.fireEvent("onIQPubSubEventSet",
				   tag.getAttribute("id"),
				   tag.getAttribute("from"),
				   ctag.getAttribute("node"),
				   ctag.getElementsByTagNameNS(this.PUBSUBEVENT_NS, "item"));
		    break;
		case "retract":
		    this.fireEvent("onIQPubSubEventDeleteItems",
				   tag.getAttribute("id"),
				   tag.getAttribute("from"),
				   ctag.getAttribute("node"),
				   map(function(data) { return data.getAttribute("id") },
				       ctag.getElementsByTagNameNS(this.PUBSUBEVENT_NS,
								   "retract")));
		}
		break;
	    case "delete":
		this.fireEvent("onIQPubSubEventDeleteNode",
			       tag.getAttribute("id"),
			       tag.getAttribute("from"),
			       ctag.getAttribute("node"));
		break;
	    }
	}
    }
}
